@@ -50,6 +50,7 @@ HTTP_METHODS_T = Literal[
 CHUNK_SIZE = 32768
 DEFAULT_USER_AGENT = f'nbxmpp/{nbxmpp.__version__}'
 SIGNAL_ACTIONS = GObject.SignalFlags.RUN_LAST | GObject.SignalFlags.ACTION
+MIN_SOUP_3_3_0 = Soup.check_version(3, 3, 0)
 
 
 class HTTPLogAdapter(logging.LoggerAdapter):
@@ -59,10 +60,10 @@ class HTTPLogAdapter(logging.LoggerAdapter):
 
 class HTTPSession:
     def __init__(self, user_agent: str = DEFAULT_USER_AGENT) -> None:
-
         self._session = Soup.Session()
         self._session.set_user_agent(user_agent)
         self._session.add_feature_by_type(Soup.ContentSniffer)
+        self._session.remove_feature_by_type(Soup.ContentDecoder)
 
     def get_soup_session(self) -> Soup.Session:
         return self._session
@@ -73,8 +74,8 @@ class HTTPSession:
 
         self._session.set_proxy_resolver(resolver)
 
-    def create_request(self) -> HTTPRequest:
-        return HTTPRequest(self)
+    def create_request(self, force_http1: bool = False) -> HTTPRequest:
+        return HTTPRequest(self, force_http1=force_http1)
 
 
 class HTTPRequest(GObject.GObject):
@@ -90,7 +91,10 @@ class HTTPRequest(GObject.GObject):
         'destroy': (SIGNAL_ACTIONS, None, ()),
     }
 
-    def __init__(self, session: HTTPSession) -> None:
+    def __init__(self,
+                 session: HTTPSession,
+                 force_http1: bool = False
+                 ) -> None:
         GObject.GObject.__init__(self)
 
         self._log = HTTPLogAdapter(log, extra={'request': self})
@@ -108,6 +112,7 @@ class HTTPRequest(GObject.GObject):
         self._is_complete = False
         self._timeout_reached = False
         self._timeout_id = None
+        self._accept_certificate_func = None
 
         self._response_body_file: Optional[Gio.File] = None
         self._response_body_data = b''
@@ -126,6 +131,8 @@ class HTTPRequest(GObject.GObject):
         self._emit_response_progress = False
 
         self._message = Soup.Message()
+        if MIN_SOUP_3_3_0:
+            self._message.set_force_http1(force_http1)
         self._user_data = None
 
         self._log.info('Created')
@@ -177,6 +184,9 @@ class HTTPRequest(GObject.GObject):
         self._log.info('Cancel requested')
         self._cancellable.cancel()
 
+    def set_accept_certificate_func(self, func: Any) -> None:
+        self._accept_certificate_func = func
+
     def set_request_body_from_path(self, content_type: str, path: Path) -> None:
         if not path.exists():
             raise ValueError('%s does not exist' % path)
@@ -278,6 +288,9 @@ class HTTPRequest(GObject.GObject):
         self._message.connect('got-body', self._on_got_body)
         self._message.connect('restarted', self._on_restarted)
         self._message.connect('finished', self._on_finished)
+        self._message.connect('got-headers', self._on_got_headers)
+        if self._accept_certificate_func is not None:
+            self._message.connect('accept-certificate', self._accept_certificate_func)
 
         soup_session = self._session.get_soup_session()
         soup_session.send_async(self._message,
@@ -301,6 +314,8 @@ class HTTPRequest(GObject.GObject):
 
     def _on_timeout(self) -> None:
         self._timeout_reached = True
+        self._timeout_id = None
+        self._set_error(HTTPRequestError.TIMEOUT)
         self.cancel()
 
     def _on_response(self,
@@ -308,6 +323,8 @@ class HTTPRequest(GObject.GObject):
                      result: Gio.AsyncResult
                      ) -> None:
 
+        self._log.info('HTTP version: %s',
+                       self._message.get_http_version().value_name)
         self._log.info('Request response received')
         try:
             self._input_stream = session.send_finish(result)
@@ -343,6 +360,7 @@ class HTTPRequest(GObject.GObject):
     def _on_bytes_read_result(self,
                               input_stream: Gio.InputStream,
                               result: Gio.AsyncResult) -> None:
+
         try:
             data = input_stream.read_bytes_finish(result)
         except GLib.Error as error:
@@ -360,10 +378,8 @@ class HTTPRequest(GObject.GObject):
             self._finish_read()
             return
 
-        length = len(bytes_)
-        self._received_size += length
-        if self._received_size > self._response_content_length:
-            self._finish_read(HTTPRequestError.CONTENT_OVERFLOW)
+        self._received_size += len(bytes_)
+        if self._check_content_overflow():
             return
 
         if self._output_stream is None:
@@ -382,12 +398,7 @@ class HTTPRequest(GObject.GObject):
                 return
 
         self._read_async()
-
-        if (self._emit_response_progress and
-                self._message.get_method() == 'GET'):
-            self.emit('response-progress',
-                      self._received_size / self._response_content_length)
-
+        self._emit_progress()
 
     def _finish_read(self, error: Optional[HTTPRequestError] = None) -> None:
         self._log.info('Finished reading')
@@ -404,11 +415,7 @@ class HTTPRequest(GObject.GObject):
                             ) -> None:
 
         # Signal is only raised when there is content in the response
-
         headers = message.get_response_headers()
-
-        self._response_content_length = headers.get_content_length()
-
         if content_type is None:
             # According to the docs, content_type is None when the sniffer
             # decides to trust the content-type sent by the server.
@@ -416,14 +423,19 @@ class HTTPRequest(GObject.GObject):
 
         self._response_content_type = content_type or ''
 
-        self._log.info('Sniffed: content-type: %s, content-length: %s',
-                       self._response_content_type,
-                       self._response_content_length)
+        self._log.info('Sniffed: content-type: %s',
+                       self._response_content_type)
 
         self.emit('content-sniffed',
                   self._response_content_length,
                   self._response_content_type)
 
+    def _on_got_headers(self, message: Soup.Message) -> None:
+        headers = message.get_response_headers()
+        self._response_content_length = headers.get_content_length()
+        self._log.info('Got Headers: content-length: %s',
+                       self._response_content_length)
+
     def _on_got_body(self, _message: Soup.Message) -> None:
         # This signal tells us that the full body was received.
         # The `finished` signal is not a sure indicator if the message body
@@ -432,11 +444,28 @@ class HTTPRequest(GObject.GObject):
         self._log.info('Body received')
         self._body_received = True
 
+    def _emit_progress(self) -> None:
+        if not self._emit_response_progress:
+            return
+
+        if not self._message.get_method() == 'GET':
+            return
+
+        self.emit('response-progress',
+                  self._received_size / self._response_content_length)
+
+    def _check_content_overflow(self) -> bool:
+        if self._received_size > self._response_content_length:
+            self._finish_read(HTTPRequestError.CONTENT_OVERFLOW)
+            return True
+        return False
+
     def _on_restarted(self, _message: Soup.Message) -> None:
         self._log.info('Restarted')
         self._body_received = False
         self._response_content_type = ''
         self._response_content_length = 0
+        self._received_size = 0
 
     def _on_finished(self, _message: Soup.Message) -> None:
         self._log.info('Message finished')
@@ -469,14 +498,17 @@ class HTTPRequest(GObject.GObject):
 
         self._set_complete()
 
+    def _set_error(self, error: HTTPRequestError) -> None:
+        self._log.info('Set Error: %s', error)
+        self._error = error
+
     def _set_failed(self, error: HTTPRequestError) -> None:
         self._log.info('Set Failed: %s', error)
         self._is_finished = True
-        if self._timeout_reached:
-            self._timeout_id = None
-            self._error = HTTPRequestError.TIMEOUT
-        else:
-            self._error = error
+
+        if self._error is None:
+            self._set_error(error)
+
         self._close_all_streams()
         self.emit('finished')
         self._cleanup()
@@ -516,7 +548,6 @@ class HTTPRequest(GObject.GObject):
         self._message.run_dispose()
 
         del self._cancellable
-        del self._session
         del self._user_data
 
         if self._timeout_id is not None:
